@@ -491,60 +491,30 @@ sub encode_json {
     require JSON::PP;
 
     my $json = JSON::PP::encode_json($data);
-    $json =~ s/([^a-zA-Z0-9_\-.])/uc sprintf("%%%02x",ord($1))/eg;
-    $json;
+    $self->uri_escape($json);
 }
 
-# TODO extract this as a module?
-sub version_to_query {
-    my($self, $module, $version) = @_;
+sub decode_json {
+    my($self, $json) = @_;
+    require JSON::PP;
 
-    require CPAN::Meta::Requirements;
+    JSON::PP::decode_json($json);
+}
 
-    my $requirements = CPAN::Meta::Requirements->new;
-    $requirements->add_string_requirement($module, $version || '0');
+sub uri_escape {
+    my($self, $fragment) = @_;
+    $fragment =~ s/([^A-Za-z0-9\-\._~])/uc sprintf("%%%02X", ord($1))/eg;
+    $fragment;
+}
 
-    my $req = $requirements->requirements_for_module($module);
-
-    if ($req =~ s/^==\s*//) {
-        return {
-            term => { 'module.version' => $req },
-        };
-    } elsif ($req !~ /\s/) {
-        return {
-            range => { 'module.version_numified' => { 'gte' => $self->numify_ver_metacpan($req) } },
-        };
-    } else {
-        my %ops = qw(< lt <= lte > gt >= gte);
-        my(%range, @exclusion);
-        my @requirements = split /,\s*/, $req;
-        for my $r (@requirements) {
-            if ($r =~ s/^([<>]=?)\s*//) {
-                $range{$ops{$1}} = $self->numify_ver_metacpan($r);
-            } elsif ($r =~ s/\!=\s*//) {
-                push @exclusion, $self->numify_ver_metacpan($r);
-            }
-        }
-
-        my @filters= (
-            { range => { 'module.version_numified' => \%range } },
-        );
-
-        if (@exclusion) {
-            push @filters, {
-                not => { or => [ map { +{ term => { 'module.version_numified' => $self->numify_ver_metacpan($_) } } } @exclusion ] },
-            };
-        }
-
-        return @filters;
+sub uri_params {
+    my($self, @params) = @_;
+    my @param_strings;
+    while (my $key = shift @params) {
+        my $value = shift @params;
+        push @param_strings, join '=', map $self->uri_escape($_), $key, $value;
     }
-}
-
-# Apparently MetaCPAN numifies devel releases by stripping _ first
-sub numify_ver_metacpan {
-    my($self, $ver) = @_;
-    $ver =~ s/_//g;
-    version->new($ver)->numify;
+    return join '&', @param_strings;
 }
 
 # version->new("1.00_00")->numify => "1.00_00" :/
@@ -553,126 +523,31 @@ sub numify_ver {
     eval version->new($ver)->numify;
 }
 
-sub maturity_filter {
-    my($self, $module, $version) = @_;
-
-    if ($version =~ /==/) {
-        # specific version: allow dev release
-        return;
-    } elsif ($self->{dev_release}) {
-        # backpan'ed dev releases are considered cancelled
-        return +{ not => { term => { status => 'backpan' } } };
-    } else {
-        return (
-            { not => { term => { status => 'backpan' } } },
-            { term => { maturity => 'released' } },
-        );
-    }
-}
-
-sub by_version {
-    my %s = qw( latest 3  cpan 2  backpan 1 );
-    $b->{_score} <=> $a->{_score} ||                             # version: higher version that satisfies the query
-    $s{ $b->{fields}{status} } <=> $s{ $a->{fields}{status} };   # prefer non-BackPAN dist
-}
-
-sub by_first_come {
-    $a->{fields}{date} cmp $b->{fields}{date};                   # first one wins, if all are in BackPAN/CPAN
-}
-
-sub by_date {
-    $b->{fields}{date} cmp $a->{fields}{date};                   # prefer new uploads, when searching for dev
-}
-
-sub find_best_match {
-    my($self, $match, $version) = @_;
-    return unless $match && @{$match->{hits}{hits} || []};
-    my @hits = $self->{dev_release}
-        ? sort { by_version || by_date } @{$match->{hits}{hits}}
-        : sort { by_version || by_first_come } @{$match->{hits}{hits}};
-    $hits[0]->{fields};
-}
-
 sub search_metacpan {
-    my($self, $module, $version) = @_;
+    my($self, $module, $version, $dev_release) = @_;
 
-    require JSON::PP;
+    my $metacpan_uri = 'http://fastapi.metacpan.org/v1/download_url/';
 
-    $self->chat("Searching $module ($version) on metacpan ...\n");
+    my $url = $metacpan_uri . $module;
 
-    my $metacpan_uri = 'http://api.metacpan.org/v0';
+    my $query = $self->uri_params(
+        ($version ? (version => $version) : ()),
+        ($dev_release ? (dev => 1) : ()),
+    );
+    $url .= '?' . $query
+        if length $query;
 
-    my @filter = $self->maturity_filter($module, $version);
+    my $dist_json = $self->get($url);
+    my $dist_meta = eval { $self->decode_json($dist_json) };
 
-    my $query = { filtered => {
-        (@filter ? (filter => { and => \@filter }) : ()),
-        query => { nested => {
-            score_mode => 'max',
-            path => 'module',
-            query => { custom_score => {
-                metacpan_script => "score_version_numified",
-                query => { constant_score => {
-                    filter => { and => [
-                        { term => { 'module.authorized' => JSON::PP::true() } },
-                        { term => { 'module.indexed' => JSON::PP::true() } },
-                        { term => { 'module.name' => $module } },
-                        $self->version_to_query($module, $version),
-                    ] }
-                } },
-            } },
-        } },
-    } };
-
-    my $module_uri = "$metacpan_uri/file/_search?source=";
-    $module_uri .= $self->encode_json({
-        query => $query,
-        fields => [ 'date', 'release', 'author', 'module', 'status' ],
-    });
-
-    my($release, $author, $module_version);
-
-    my $module_json = $self->get($module_uri);
-    my $module_meta = eval { JSON::PP::decode_json($module_json) };
-    my $match = $self->find_best_match($module_meta);
-    if ($match) {
-        $release = $match->{release};
-        $author = $match->{author};
-        my $module_matched = (grep { $_->{name} eq $module } @{$match->{module}})[0];
-        $module_version = $module_matched->{version};
-    }
-
-    unless ($release) {
-        $self->chat("! Could not find a release matching $module ($version) on MetaCPAN.\n");
-        return;
-    }
-
-    my $dist_uri = "$metacpan_uri/release/_search?source=";
-    $dist_uri .= $self->encode_json({
-        filter => { and => [
-            { term => { 'release.name' => $release } },
-            { term => { 'release.author' => $author } },
-        ]},
-        fields => [ 'download_url', 'stat', 'status' ],
-    });
-
-    my $dist_json = $self->get($dist_uri);
-    my $dist_meta = eval { JSON::PP::decode_json($dist_json) };
-
-    if ($dist_meta) {
-        $dist_meta = $dist_meta->{hits}{hits}[0]{fields};
-    }
     if ($dist_meta && $dist_meta->{download_url}) {
         (my $distfile = $dist_meta->{download_url}) =~ s!.+/authors/id/!!;
         local $self->{mirrors} = $self->{mirrors};
-        if ($dist_meta->{status} eq 'backpan') {
-            $self->{mirrors} = [ 'http://backpan.perl.org' ];
-        } elsif ($dist_meta->{stat}{mtime} > time()-24*60*60) {
-            $self->{mirrors} = [ 'http://cpan.metacpan.org' ];
-        }
-        return $self->cpan_module($module, $distfile, $module_version);
+        $self->{mirrors} = [ 'http://cpan.metacpan.org' ];
+        return $self->cpan_module($module, $distfile, $dist_meta->{version});
     }
 
-    $self->diag_fail("Finding $module on metacpan failed.");
+    $self->chat("! Could not find a release matching $module".($version?" ($version)":'')." on MetaCPAN.\n");
     return;
 }
 
@@ -682,8 +557,8 @@ sub search_database {
     my $found;
 
     if ($self->{dev_release} or $self->{metacpan}) {
-        $found = $self->search_metacpan($module, $version)   and return $found;
-        $found = $self->search_cpanmetadb($module, $version) and return $found;
+        $found = $self->search_metacpan($module, $version, $self->{dev_release})   and return $found;
+        $found = $self->search_cpanmetadb($module, $version, $self->{dev_release}) and return $found;
     } else {
         $found = $self->search_cpanmetadb($module, $version) and return $found;
         $found = $self->search_metacpan($module, $version)   and return $found;
@@ -691,20 +566,20 @@ sub search_database {
 }
 
 sub search_cpanmetadb {
-    my($self, $module, $version) = @_;
+    my($self, $module, $version, $dev_release) = @_;
 
 
     $self->chat("Searching $module ($version) on cpanmetadb ...\n");
 
     if ($self->with_version_range($version)) {
-        return $self->search_cpanmetadb_history($module, $version);
+        return $self->search_cpanmetadb_history($module, $version, $dev_release);
     } else {
-        return $self->search_cpanmetadb_package($module, $version);
+        return $self->search_cpanmetadb_package($module, $version, $dev_release);
     }
 }
 
 sub search_cpanmetadb_package {
-    my($self, $module, $version) = @_;
+    my($self, $module, $version, $dev_release) = @_;
 
     require CPAN::Meta::YAML;
 
@@ -1816,7 +1691,7 @@ sub verify_signature {
         $self->diag_ok("Verified OK");
         return 1;
     } else {
-        $self->diag_fail("SIGNATURE verificaion for $dist->{filename} failed\n");
+        $self->diag_fail("SIGNATURE verification for $dist->{filename} failed\n");
         return;
     }
 }
@@ -1870,9 +1745,9 @@ sub resolve_name {
 }
 
 sub cpan_module {
-    my($self, $module, $dist, $version) = @_;
+    my($self, $module, $dist_file, $version) = @_;
 
-    my $dist = $self->cpan_dist($dist);
+    my $dist = $self->cpan_dist($dist_file);
     $dist->{module} = $module;
     $dist->{module_version} = $version if $version && $version ne 'undef';
 
@@ -3125,9 +3000,13 @@ sub init_tools {
             my $opt = $self->{verbose} ? '' : '-q';
             my(undef, $root, @others) = `$unzip -t $zipfile`
                 or return undef;
-
-            chomp $root;
-            $root =~ s{^\s+testing:\s+([^/]+)/.*?\s+OK$}{$1};
+            FILE: {
+                chomp $root;
+                if ($root !~ s{^\s+testing:\s+([^/]+)/.*?\s+OK$}{$1}) {
+                    $root = shift(@others);
+                    redo FILE if $root;
+                }
+            }
 
             system "$unzip $opt $zipfile";
             return $root if -d $root;
